from django import forms
from django.core.exceptions import ValidationError
from django.utils.formats import date_format
from django.utils.translation import gettext_lazy as _
from django_scopes.forms import SafeModelChoiceField
from i18nfield.forms import I18nFormField, I18nTextarea, I18nTextInput
from pretix.base.email import get_available_placeholders
from pretix.base.forms import PlaceholderValidator, SettingsForm
from pretix.base.models import Item, Order

from .models import ItemConfig, LinkedOrderPosition


def can_use_juvare_api(event):
    try:
        import pretix_juvare_notify  # noqa
    except ImportError:
        return False
    # Is the plugin active
    if "pretix_juvare_notify" not in event.plugins:
        return False
    # Is the plugin configured
    if not event.settings.juvare_client_secret:
        return False
    return True


class ForeignKeyRawIdWidget(forms.TextInput):
    pass


class ItemConfigForm(forms.ModelForm):
    class Meta:
        model = ItemConfig
        fields = ["days", "max_days", "event", "second_item"]
        widgets = {"second_item": ForeignKeyRawIdWidget}
        exclude = []
        field_classes = {
            "event": SafeModelChoiceField,
            "second_item": SafeModelChoiceField,
        }

    def __init__(self, *args, **kwargs):
        self.event = kwargs.pop("event")
        super().__init__(*args, **kwargs)
        self.fields["event"].queryset = self.event.organizer.events.filter(
            has_subevents=True
        ).order_by("date_from")
        self.fields["second_item"].queryset = Item.objects.filter(
            event__organizer=self.event.organizer
        )
        self.fields["days"].required = False
        self.fields["days"].widget.is_required = False

    def clean(self):
        data = super().clean()
        if data.get("days"):
            if not self.event.has_subevents:
                raise ValidationError("This may only be used on event series.")
        if data.get("event"):
            target_event = data["event"]
        else:
            target_event = self.event

        if data.get("second_item"):
            if data["second_item"].event != target_event:
                raise ValidationError(
                    _("The product ID you entered is for the wrong event.")
                )
        elif target_event != self.event:
            has_item = (
                len(
                    [
                        n
                        for n in target_event.items.all()
                        if (n.internal_name or str(n.name))
                        == (
                            self.instance.item.internal_name
                            or str(self.instance.item.name)
                        )
                    ]
                )
                == 1
            )
            if not has_item:
                raise ValidationError(
                    _(
                        "You selected an event series that does not have product with the same name as this "
                        "product."
                    )
                )
        return data

    def save(self, commit=True):
        if self.cleaned_data["days"] is None:
            if self.instance.pk:
                self.instance.delete()
            else:
                return
        else:
            return super().save(commit=commit)


class AutoschedSettingsForm(SettingsForm):
    vacc_autosched_checkin = forms.BooleanField(
        label=_("Auto-schedule second dose after check-in"),
        required=False,
    )
    vacc_autosched_mail = forms.BooleanField(
        label=_("Send email if second dose has been scheduled"),
        required=False,
    )

    vacc_autosched_subject = I18nFormField(
        label=_("Email Subject"),
        required=True,
        widget=I18nTextInput,
    )

    vacc_autosched_body = I18nFormField(
        label=_("Email Body"),
        required=True,
        widget=I18nTextarea,
    )

    vacc_autosched_sms = forms.BooleanField(
        label=_("Send an SMS if second dose has been scheduled"),
        help_text=_("Uses the Juvare Notify API"),
        required=False,
    )
    vacc_autosched_sms_text = I18nFormField(
        label=_("SMS Body"),
        required=True,
        widget=I18nTextarea,
    )
    vacc_autosched_self_service = forms.BooleanField(
        label=_("Self-service for second doses"),
        help_text=_(
            "Allows customers to schedule their second dose via the /2nd page."
        ),
        required=False,
    )
    vacc_autosched_self_service_info = I18nFormField(
        label=_("Self service introduction"),
        help_text=_(
            "This text will be shown on the general self-service page, where customers enter their order code. You can use Markdown here."
        ),
        required=False,
        widget=I18nTextarea,
    )
    vacc_autosched_self_service_order_info = I18nFormField(
        label=_("Self service order information"),
        help_text=_(
            "This text will be shown on the page where customers order their second appointment. You can use Markdown here."
        ),
        required=False,
        widget=I18nTextarea,
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.event = kwargs.pop("obj")

        self._set_field_placeholders(
            "vacc_autosched_subject", ["event"], ["{scheduled_datetime}"]
        )
        self._set_field_placeholders(
            "vacc_autosched_body", ["event"], ["{scheduled_datetime}"]
        )
        if not can_use_juvare_api(self.event):
            self.fields.pop("vacc_autosched_sms")
            self.fields.pop("vacc_autosched_sms_text")

    def _set_field_placeholders(self, fn, base_parameters, extras=[]):
        phs = [
            "{%s}" % p
            for p in sorted(
                get_available_placeholders(self.event, base_parameters).keys()
            )
        ] + extras
        ht = _("Available placeholders: {list}").format(list=", ".join(phs))
        if self.fields[fn].help_text:
            self.fields[fn].help_text += " " + str(ht)
        else:
            self.fields[fn].help_text = ht
        self.fields[fn].validators.append(PlaceholderValidator(phs))


class SecondDoseCodeForm(forms.Form):
    order = forms.CharField(label=_("Order code"))

    def __init__(self, event, *args, **kwargs):
        self.event = event
        super().__init__(*args, **kwargs)

    def clean_order(self):
        code = self.cleaned_data.get("order")
        qs = self.event.orders.filter(status=Order.STATUS_PAID)
        order = qs.filter(code__iexact=code).first()
        if not order:
            order = qs.filter(secret__iexact=code).first()
        if not order:
            order = qs.filter(all_positions__secret__iexact=code).first()

        if not order:
            raise forms.ValidationError(
                _(
                    "We were unable to find a valid ticket with this code, please try again."
                )
            )

        positions = list(order.positions.all())

        if len(positions) != 1:
            raise forms.ValidationError(
                _(
                    "This functionality is currently not available if multiple tickets have been created in one booking."
                )
            )

        position = positions[0]

        lop = LinkedOrderPosition.objects.filter(base_position=position).first()
        if lop:
            raise forms.ValidationError(
                _(
                    "A second appointment has already been scheduled for {datetime}. The ticket has been sent to "
                    "you via email to the address used for your first booking."
                ).format(
                    datetime=date_format(
                        lop.child_position.subevent.date_from.astimezone(
                            self.event.timezone
                        ),
                        "DATETIME_FORMAT",
                    )
                )
            )

        return order


class SecondDoseOrderForm(forms.Form):
    def __init__(self, *args, position, available_subevents, event, **kwargs):
        self.position = position
        super().__init__(*args, **kwargs)

        self.subevents = available_subevents
        self.fields["subevent"] = forms.ModelChoiceField(
            queryset=event.subevents.filter(
                pk__in=[e.pk for e in available_subevents]
            ).order_by("date_from"),
            label=_("Date"),
            widget=forms.RadioSelect,
        )



# Generated by Django 3.0.12 on 2021-03-10 10:17

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("pretixbase", "0175_orderrefund_comment"),
    ]

    operations = [
        migrations.CreateModel(
            name="ItemConfig",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ("days", models.IntegerField()),
                (
                    "event",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="pretixbase.Event",
                    ),
                ),
                (
                    "item",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="vacc_sautosched_config",
                        to="pretixbase.Item",
                    ),
                ),
            ],
        ),
    ]



from django.db import models
from django.utils.translation import gettext_lazy as _
from pretix.base.models import OrderPosition


class ItemConfig(models.Model):
    item = models.OneToOneField(
        "pretixbase.Item",
        related_name="vacc_autosched_config",
        on_delete=models.CASCADE,
    )
    days = models.IntegerField(
        verbose_name=_("Scheduling of second dose: Number of days"),
        help_text=_("Keep empty to turn feature off"),
    )
    max_days = models.IntegerField(
        verbose_name=_("Scheduling of second dose: Maximum number of days"),
        help_text=_("Used in the self-service functionality."),
        null=True,
        blank=True,
    )
    event = models.ForeignKey(
        "pretixbase.Event",
        on_delete=models.CASCADE,
        null=True,
        blank=True,
        verbose_name=_("Scheduling of second dose: Event series to choose date from"),
        help_text=_("If empty, the current series will be used"),
    )
    second_item = models.ForeignKey(
        "pretixbase.Item",
        on_delete=models.CASCADE,
        null=True,
        blank=True,
        verbose_name=_("Scheduling of second dose: Product name for second dose"),
        help_text=_(
            "If empty, a product with the same name as the current one will be chosen"
        ),
    )


class LinkedOrderPosition(models.Model):
    base_position = models.ForeignKey(
        OrderPosition, related_name="vacc_autosched_linked", on_delete=models.PROTECT
    )
    child_position = models.OneToOneField(
        OrderPosition, related_name="vacc_autosched_link", on_delete=models.PROTECT
    )



from django.conf.urls import url

from . import api
from .views import SelfServiceBookingView, SelfServiceIndexView, SettingsView

urlpatterns = [
    url(
        r"^control/event/(?P<organizer>[^/]+)/(?P<event>[^/]+)/settings/vacc_autosched/$",
        SettingsView.as_view(),
        name="settings",
    ),
    url(
        r"^api/v1/organizers/(?P<organizer>[^/]+)/events/(?P<event>[^/]+)/items/(?P<item>[^/]+)/vacc_autosched/$",
        api.ItemView.as_view(),
        name="api.item",
    ),
]

event_patterns = [
    url(
        r"^2nd/(?P<order>[^/]+)/$",
        SelfServiceBookingView.as_view(),
        name="second.booking",
    ),
    url(
        r"^2nd/?$",
        SelfServiceIndexView.as_view(),
        name="second.index",
    ),
]



import copy
from django.dispatch import receiver
from django.urls import resolve, reverse
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_noop, ugettext_lazy as _
from i18nfield.rest_framework import I18nField
from i18nfield.strings import LazyI18nString
from pretix.base.settings import settings_hierarkey
from pretix.base.signals import (
    api_event_settings_fields,
    checkin_created,
    event_copy_data,
    item_copy_data,
    logentry_display,
)
from pretix.control.signals import item_forms, nav_event_settings
from rest_framework import serializers

from pretix_vacc_autosched.tasks import schedule_second_dose

from .forms import ItemConfigForm
from .models import ItemConfig


@receiver(nav_event_settings, dispatch_uid="vacc_autosched_nav")
def navbar_info(sender, request, **kwargs):
    url = resolve(request.path_info)
    if not request.user.has_event_permission(
        request.organizer, request.event, "can_change_event_settings", request=request
    ):
        return []
    return [
        {
            "label": _("Vaccination scheduling"),
            "url": reverse(
                "plugins:pretix_vacc_autosched:settings",
                kwargs={
                    "event": request.event.slug,
                    "organizer": request.organizer.slug,
                },
            ),
            "active": url.namespace == "plugins:pretix_vacc_autosched",
        }
    ]


@receiver(item_forms, dispatch_uid="vacc_autosched_item_forms")
def control_item_forms(sender, request, item, **kwargs):
    try:
        inst = ItemConfig.objects.get(item=item)
    except ItemConfig.DoesNotExist:
        inst = ItemConfig(item=item)
    return ItemConfigForm(
        instance=inst,
        event=sender,
        data=(request.POST if request.method == "POST" else None),
        prefix="vacc_autosched_conf",
    )


@receiver(item_copy_data, dispatch_uid="vacc_autosched_item_copy")
def copy_item(sender, source, target, **kwargs):
    try:
        inst = ItemConfig.objects.get(item=source)
        inst = copy.copy(inst)
        inst.pk = None
        inst.item = target
        inst.save()
    except ItemConfig.DoesNotExist:
        pass


@receiver(signal=event_copy_data, dispatch_uid="vacc_autosched_copy_data")
def event_copy_data_receiver(sender, other, question_map, item_map, **kwargs):
    for ip in ItemConfig.objects.filter(item__event=other):
        ip = copy.copy(ip)
        ip.pk = None
        ip.item = item_map[ip.item_id]
        ip.save()


@receiver(signal=checkin_created, dispatch_uid="vacc_autosched_checkin_created")
def checkin_created_receiver(sender, checkin, **kwargs):
    if checkin.list.name.startswith("Print"):
        return  # ignore, not a "real" check-in
    if not sender.settings.vacc_autosched_checkin:
        return  # ignore, handled manually

    try:
        ItemConfig.objects.get(item=checkin.position.item)
    except ItemConfig.DoesNotExist:
        return  # ignore, not configured

    schedule_second_dose.apply_async(
        args=(
            sender.pk,
            checkin.position.pk,
        )
    )


@receiver(
    signal=api_event_settings_fields,
    dispatch_uid="vacc_autosched_api_event_settings_fields",
)
def recv_api_event_settings_fields(sender, **kwargs):
    return {
        "vacc_autosched_checkin": serializers.BooleanField(required=False),
        "vacc_autosched_mail": serializers.BooleanField(required=False),
        "vacc_autosched_subject": I18nField(required=False),
        "vacc_autosched_body": I18nField(required=False),
        "vacc_autosched_sms": serializers.BooleanField(required=False),
        "vacc_autosched_sms_text": I18nField(required=False),
        "vacc_autosched_self_service": serializers.BooleanField(required=False),
        "vacc_autosched_self_service_info": I18nField(required=False),
        "vacc_autosched_self_service_order_info": I18nField(required=False),
    }


@receiver(signal=logentry_display, dispatch_uid="vacc_autosched_logentry_display")
def badges_logentry_display(sender, logentry, **kwargs):
    if not logentry.action_type.startswith("pretix_vacc_autosched."):
        return

    d = logentry.parsed_data
    if logentry.action_type == "pretix_vacc_autosched.failed":
        return _("Automatic scheduling of second dose failed: {reason}").format(
            reason=d.get("reason")
        )
    if logentry.action_type == "pretix_vacc_autosched.scheduled":
        url = reverse(
            "control:event.order",
            kwargs={
                "event": d.get("event_slug"),
                "organizer": logentry.organizer.slug,
                "code": d.get("order"),
            },
        )
        return mark_safe(
            _("Second dose scheduled automatically: {order}").format(
                order='<a href="{}">{}</a>'.format(url, d.get("order")),
            )
        )
    if logentry.action_type == "pretix_vacc_autosched.created":
        url = reverse(
            "control:event.order",
            kwargs={
                "event": d.get("event_slug"),
                "organizer": logentry.organizer.slug,
                "code": d.get("order"),
            },
        )
        return mark_safe(
            _(
                "This order has been scheduled as the second dose for order {order}"
            ).format(
                order='<a href="{}">{}</a>'.format(url, d.get("order")),
            )
        )


settings_hierarkey.add_default("vacc_autosched_mail", False, bool)
settings_hierarkey.add_default("vacc_autosched_self_service", False, bool)
settings_hierarkey.add_default(
    "vacc_autosched_self_service_info",
    "",
    LazyI18nString,
)
settings_hierarkey.add_default(
    "vacc_autosched_self_service_order_info",
    "",
    LazyI18nString,
)
settings_hierarkey.add_default(
    "vacc_autosched_subject",
    LazyI18nString.from_gettext(gettext_noop("Your second dose: {scheduled_datetime}")),
    LazyI18nString,
)
settings_hierarkey.add_default(
    "vacc_autosched_body",
    LazyI18nString.from_gettext(
        gettext_noop(
            "Hello,\n\n"
            "we scheduled your second dose for {scheduled_datetime}.\n\n"
            "Please find additional information in your ticket attached.\n\n"
            "Best regards,\n"
            "Your {event} team"
        )
    ),
    LazyI18nString,
)
settings_hierarkey.add_default("vacc_autosched_sms", False, bool)
settings_hierarkey.add_default(
    "vacc_autosched_sms_text",
    LazyI18nString.from_gettext(
        gettext_noop(
            "Hello,\n\n"
            "we scheduled your second dose for {scheduled_datetime}.\n\n"
            "Please find additional information here: {url}.\n\n"
        )
    ),
    LazyI18nString,
)
settings_hierarkey.add_default("vacc_autosched_checkin", True, bool)



import logging
from datetime import datetime, timedelta
from decimal import Decimal
from django.db import transaction
from django.db.models import Q
from django.utils.formats import date_format
from django.utils.timezone import make_aware, now
from django.utils.translation import ugettext_lazy as _
from pretix.base.email import get_email_context
from pretix.base.i18n import language
from pretix.base.models import Order, OrderPosition, Quota
from pretix.base.services.locking import LockTimeoutException
from pretix.base.services.mail import SendMailException, TolerantDict
from pretix.base.services.tasks import EventTask
from pretix.base.signals import order_paid, order_placed
from pretix.celery_app import app

from pretix_vacc_autosched.forms import can_use_juvare_api
from pretix_vacc_autosched.models import LinkedOrderPosition

logger = logging.getLogger(__name__)


def get_for_other_event(op, event, prefer_second_item):
    logger.info(f"SECOND DOSE: Looking up item for event {event.slug}, prefer item {prefer_second_item}")
    if prefer_second_item:
        target_item = prefer_second_item
        if event != target_item.event:
            logger.info(f"SECOND DOSE: Abort because preferred item is for event {target_item.event.slug}")
            return None, None
    elif op.order.event == event:
        logger.info(f"SECOND DOSE: Choose same item because of same event")
        return op.item, op.variation
    else:
        possible_items = [
            n
            for n in event.items.all()
            if (n.internal_name or str(n.name))
            == (op.item.internal_name or str(op.item.name))
        ]
        if len(possible_items) != 1:
            op.order.log_action(
                "pretix_vacc_autosched.failed",
                data={
                    "reason": _("No product found"),
                    "position": op.pk,
                },
            )
            logger.info(f"SECOND DOSE: Possible items by name: {repr([n.pk for n in possible_items])}")
            return None, None

        target_item = possible_items[0]

    if op.variation or target_item.variations.exists():
        possible_variations = [
            n
            for n in target_item.variations.all()
            if str(n.value) == (str(op.variation.value) if op.variation else None)
        ]
        if len(possible_variations) != 1:
            logger.info(f"SECOND DOSE: Possible variations by name: {repr([n.pk for n in possible_variations])}")
            op.order.log_action(
                "pretix_vacc_autosched.failed",
                data={
                    "reason": _("No product variation found"),
                    "position": op.pk,
                },
            )
            return None, None
        target_var = possible_variations[0]
    else:
        target_var = None
    return target_item, target_var


@app.task(base=EventTask, bind=True, max_retries=5, default_retry_delay=60)
def schedule_second_dose(self, event, op):
    op = OrderPosition.objects.select_related(
        "item", "variation", "subevent", "order"
    ).get(pk=op)

    logger.info(f"SECOND DOSE: Scheduling started for {op.order.code}")

    if LinkedOrderPosition.objects.filter(
        Q(base_position=op) | Q(child_position=op)
    ).exists():
        logger.info("SECOND DOSE: Scheduling aborted, seond dose already booked")
        return

    itemconf = op.item.vacc_autosched_config
    earliest_date = make_aware(
        datetime.combine(
            op.subevent.date_from.astimezone(event.timezone).date()
            + timedelta(days=itemconf.days),
            op.subevent.date_from.astimezone(event.timezone).time(),
        ),
        event.timezone,
    )

    target_event = itemconf.event or event
    target_item, target_var = get_for_other_event(
        op, target_event, itemconf.second_item
    )

    logger.info(f"SECOND DOSE: date after {earliest_date}, target_event {target_event.slug}, target_item {target_item.pk}, target_variation {target_var.pk if target_var else None}")

    if target_item is None:
        return

    for i in range(250):  # max number of subevents to check
        subevent = (
            target_event.subevents.filter(
                date_from__gte=earliest_date,
            )
            .order_by("date_from")
            .first()
        )
        if not subevent:
            logger.info(f"SECOND DOSE: no time slot found after {earliest_date}")
            op.order.log_action(
                "pretix_vacc_autosched.failed",
                data={
                    "reason": _("No available time slot found"),
                    "position": op.pk,
                },
            )
            return

        try:
            order = book_second_dose(
                op=op,
                item=target_item,
                variation=target_var,
                subevent=subevent,
                original_event=event,
            )
            if not order:
                earliest_date += timedelta(minutes=1)
                continue
            else:
                return
        except LockTimeoutException:
            self.retry()

    logger.info(f"SECOND DOSE: no available time slot found after 250 tries")
    op.order.log_action(
        "pretix_vacc_autosched.failed",
        data={
            "reason": _("No available time slot found"),
            "position": op.pk,
        },
    )
    return


def book_second_dose(*, op, item, variation, subevent, original_event):
    event = item.event
    with event.lock(), transaction.atomic():
        avcode, avnr = item.check_quotas(subevent=subevent, fail_on_no_quotas=True)
        if avcode != Quota.AVAILABILITY_OK:
            logger.info(f"SECOND DOSE: cannot use slot {subevent.pk}, sold out")
            # sold out, look for next one
            return

        childorder = Order.objects.create(
            event=event,
            status=Order.STATUS_PAID,
            require_approval=False,
            testmode=op.order.testmode,
            email=op.order.email,
            phone=op.order.phone,
            locale=op.order.locale,
            expires=now() + timedelta(days=30),
            total=Decimal("0.00"),
            expiry_reminder_sent=True,
            sales_channel=op.order.sales_channel,
            comment="Auto-generated through scheduling from order {}".format(
                op.order.code
            ),
            meta_info=op.order.meta_info,
        )
        op.order.log_action(
            "pretix_vacc_autosched.scheduled",
            data={
                "position": op.pk,
                "event": event.pk,
                "event_slug": event.slug,
                "order": childorder.code,
            },
        )
        childorder.log_action(
            "pretix_vacc_autosched.created",
            data={
                "event": original_event.pk,
                "event_slug": original_event.slug,
                "order": op.order.code,
            },
        )
        childorder.log_action(
            "pretix.event.order.placed", data={"source": "vacc_autosched"}
        )
        childpos = childorder.positions.create(
            positionid=1,
            tax_rate=Decimal("0.00"),
            tax_rule=None,
            tax_value=Decimal("0.00"),
            subevent=subevent,
            item=item,
            variation=variation,
            price=Decimal("0.00"),
            attendee_name_cached=op.attendee_name_cached,
            attendee_name_parts=op.attendee_name_parts,
            attendee_email=op.attendee_email,
            company=op.company,
            street=op.street,
            zipcode=op.zipcode,
            city=op.city,
            country=op.country,
            state=op.state,
            addon_to=None,
            voucher=None,
            meta_info=op.meta_info,
        )
        for answ in op.answers.all():
            q = childorder.event.questions.filter(
                identifier=answ.question.identifier
            ).first()
            if not q:
                continue
            childansw = childpos.answers.create(
                question=q, answer=answ.answer, file=answ.file
            )
            if answ.options.all():
                childopts = list(
                    q.options.filter(
                        identifier__in=[o.identifier for o in answ.options.all()]
                    )
                )
                if childopts:
                    childansw.options.add(*childopts)

        LinkedOrderPosition.objects.create(base_position=op, child_position=childpos)
        childorder.create_transactions(is_new=True)
    order_placed.send(event, order=childorder)
    order_paid.send(event, order=childorder)

    if original_event.settings.vacc_autosched_mail:
        with language(childorder.locale, original_event.settings.region):
            email_template = original_event.settings.vacc_autosched_body
            email_subject = str(original_event.settings.vacc_autosched_subject)

            email_context = get_email_context(event=childorder.event, order=childorder)
            email_context["scheduled_datetime"] = date_format(
                subevent.date_from.astimezone(original_event.timezone),
                "SHORT_DATETIME_FORMAT",
            )
            try:
                childorder.send_mail(
                    email_subject,
                    email_template,
                    email_context,
                    "pretix.event.order.email.order_placed",
                    attach_tickets=True,
                    attach_ical=event.settings.mail_attach_ical,
                )
            except SendMailException:
                logger.exception("Order approved email could not be sent")
    if (
        original_event.settings.vacc_autosched_sms
        and can_use_juvare_api(original_event)
        and childorder.phone
    ):
        with language(childorder.locale, original_event.settings.region):
            from pretix_juvare_notify.tasks import juvare_send_text

            template = original_event.settings.vacc_autosched_sms_text
            context = get_email_context(event=childorder.event, order=childorder)
            context["scheduled_datetime"] = date_format(
                subevent.date_from.astimezone(original_event.timezone),
                "SHORT_DATETIME_FORMAT",
            )
            message = str(template).format_map(TolerantDict(context))
            juvare_send_text.apply_async(
                kwargs={
                    "text": message,
                    "to": childorder.phone,
                    "event": original_event.pk,
                }
            )
    logger.info(f"SECOND DOSE: done, created order {childorder.code}")
    return childorder



# Generated by Django 3.0.11 on 2021-03-26 10:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("pretix_vacc_autosched", "0002_auto_20210310_1210"),
    ]

    operations = [
        migrations.AddField(
            model_name="itemconfig",
            name="max_days",
            field=models.IntegerField(null=True),
        ),
    ]
